import streamlit as st
import numpy as np
import plotly.graph_objects as go
from animation import Draw2D, Draw3D, Traces2d, lorenz_Traces


option = st.sidebar.selectbox('Which animation do you want to draw?', ['2d','3d'])
'You have selected:', option


if option == "3d":
    k = st.sidebar.slider('how many lines?', 1, 10, 5)
    all_step = st.sidebar.slider('how many steps?', 500, 5000, 900)
    total_frames = st.sidebar.slider('total frames', 100, 1000, 200)
    period = st.sidebar.slider('period', 1, 100, 5)
    period /= 100
    @st.cache(suppress_st_warning=True)
    def drawing(all_step, k, total_frames, period):
        draw3d = Draw3D()
        draw3d.Traces = lorenz_Traces(x0=5, y0=6, z0=10, lambd=10,
                                      k=k, all_step=all_step)
        fig = draw3d.anima(total_frames=total_frames, period=period)
        return fig

    if st.checkbox("3D rendering"):
        fig = drawing(all_step, k, total_frames, period)
        st.plotly_chart(fig)


if option == "2d":

    all_step = st.sidebar.slider('all step?', 1000, 6000, 1500)
    total_frames = st.sidebar.slider('total frames', 100, 1000, 200)
    period = st.sidebar.slider('period', 1, 100, 5)
    period /= 100
    @st.cache(suppress_st_warning=True)
    def drawing(all_step, total_frames, period, width, height):
        draw2d = Draw2D()
        x = np.linspace(0, 20, all_step)[..., None]
        y = np.sin(x)
        Trace = []
        Trace.append(np.concatenate((x, y), axis=1))

        draw2d.Traces = Trace
        fig = draw2d.anima(total_frames=total_frames, period=period, width=990, height=400)
        return fig

    if st.checkbox("2D rendering"):
        fig = drawing(all_step=all_step, total_frames=total_frames, period=period, width=990, height=400)
        st.plotly_chart(fig)


import numpy as np
from scipy import integrate


def f(xyz,t, sigma,beta,lambd):
    x, y, z = xyz
    return [sigma * (y-x), beta * x - y - x*z, -lambd* z+ x*y]

def Trace(**kwargs):
    """
    :param `t` : time, default np.linspace(0, 20, 5000)
    :param `xyz` : initial xyz value, default [1,1,1]
    :param `sigma` default 10
    :param `beta` default 28
    :param `lambd` default 8/3
    """
    t = kwargs.get('t', np.linspace(0, 20, 5000))
    xyz = kwargs.get('xyz', [1,1,1])
    sigma=kwargs.get('sigma', 10)
    beta =kwargs.get('beta', 28)
    lambd=kwargs.get('lambd', 8/3)

    trace = integrate.odeint(f, xyz, t, args=(sigma, beta, lambd))
    return trace



from IPython.core.interactiveshell import InteractiveShell
# import imp
import importlib

def output(flag='all'):
    """
    'all', 'last', 'last_expr' or 'none'
    """
    InteractiveShell.ast_node_interactivity = flag

def reload(module):
    # imp.reload(module)
    importlib.reload(module)




from enum import IntEnum
import time
import datetime
from datetime import timedelta, timezone
import numpy as np
from .time import beijing
from colorama import init, Fore, Back, Style


class Fc(IntEnum):
    """Foreground color"""
    black = 30
    red = 31
    green = 32
    yellow = 33
    blue = 34
    magenta = 35
    cyan = 36
    white = 37
class Bc(IntEnum):
    """Background color"""
    black = 40
    red = 41
    green = 42
    yellow = 43
    blue = 44
    magenta = 45
    cyan = 46
    white = 47
class Disp(IntEnum):
    """Effect of display"""
    default = 0
    highlight = 1
    underline = 4
    twinkle = 5 # 闪烁
    reverse = 6 # 反白显示
    invisible = 8

def cprint(string, fc=Fc.cyan, bg=False, bc=Bc.black, coverage='\r'):

    if bg:
        print(f'{coverage}\033[{Disp.highlight};{fc};{bc}m{string}\033[0m', end='', flush=True)
    else:
        print(f'{coverage}\033[{Disp.highlight};{fc}m{string}\033[0m', end='', flush=True)

def get_bar(percent):
    unit_percent = 0.034
    total_space = 30
    n_sign1, mod_sign1 = divmod(percent, unit_percent)
    N1 = int(n_sign1)
    sign1 = "█" * N1
    N0 = int((mod_sign1/unit_percent) * (total_space - N1))
    sign0 = '>' * N0
    SIGN = '|' + sign1 + sign0 + (total_space- N1 - N0-1) * ' ' + '|'
    return SIGN, N1

def get_color(N_color, update=True, COLOR=[0]):
    if update == True or COLOR[0] == 0:
        color_list = ['CYAN', 'GREEN', 'RED', 'YELLOW', 'RESET', \
                      'LIGHTGREEN_EX',  'LIGHTRED_EX', \
                      'LIGHTYELLOW_EX',  'LIGHTBLACK_EX', 'LIGHTBLUE_EX', 'LIGHTCYAN_EX']
        # 'LIGHTMAGENTA_EX', 'MAGENTA', 'BLUE',

        color = [Fore.LIGHTCYAN_EX]  # Manually specify the first color
        for i in range(N_color - 2):
            color.append(eval("Fore." + np.random.choice(color_list)))
        color.append(Fore.LIGHTBLUE_EX)  # and the last color
        COLOR[0] = color

        return COLOR[0]
    else:
        return COLOR[0]

def bar(index, total_size, color='constant_random', first_time=[time.time()], flag_list=[1000], COLOR=[0]):
    """
    :arg color: options  'constant_random', 'update_random', 'reset'
    """
    # init(autoreset=True)
    _index = index + 1
    percent = (_index)/total_size # index+1 for index start from 0
    cost_time = time.time() - first_time[0]
    total_time = cost_time/percent
    remain_time = int(total_time - cost_time)
    remain_time = timedelta(seconds=remain_time)
    total_time = timedelta(seconds=int(total_time))
    ETC_1 = f"{remain_time}|{total_time} "
    ETC_2 = f"ETC: {(datetime.datetime.now() + remain_time).strftime('%m-%d %H:%M:%S')}"

    SIGN, N1 = get_bar(percent)

    if color == "update_random":
        if flag_list[0] != N1:
            flag_list[0] = N1
            COLOR[0] = get_color(N_color=4, update=True)
            [color1, color2, color3, color4] = COLOR[0]
        else:
            [color1, color2, color3, color4] = COLOR[0]

        print(color1 + f"\r{percent * 100: >6.2f}% ", color2 + SIGN, color3 + f"{ETC_1}", color4 + f"{ETC_2}", Fore.RESET, end='',
              flush=True)
    elif color == 'constant_random':
        [color1, color2, color3, color4] = get_color(N_color=4, update=False)
        print(f"\r{color1}{percent * 100: >6.2f}% "+ color2 + SIGN + color3 + f"{ETC_1}"+ color4 + f"{ETC_2}"+ Fore.RESET, end='',
              flush=True)
    elif color == 'reset':

        print(f"\r{percent * 100: >6.2f}% "+ SIGN + f"{ETC_1} {ETC_2}", end='', flush=True)
    else:
        raise ValueError("Invalid input!")

    if _index == total_size:
        print('\n')



class probar:
    """
    Simple progress bar display, to instead of tqdm.
    """

    def __init__(self, iterable, total_steps=None):
        self.iterable = iterable
        self.t0 = time.time()

        if hasattr(iterable, '__len__'):
            self.total_steps = len(iterable)
        else:
            self.total_steps = total_steps
            if self.total_steps == None:
                raise ValueError(f'{iterable} has no __len__ attr, use total_steps param')

    def __iter__(self):
        for idx, i in enumerate(self.iterable):
            c = idx + 1

            if idx == 0:
                print(f"\r{0:.2f}% \t  {0:.1f}|{np.inf:.1f}s ", end='', flush=True)
                d_percent = 0.01
            else:
                # percent = self.c / self.total_steps
                percent = c / self.total_steps
                PERCENT = percent * 100

                if PERCENT >= d_percent:
                    d_percent += 0.01
                    cost_time = time.time() - self.t0
                    total_time = cost_time / percent

                    remain_time = int(total_time - cost_time)
                    remain_time = datetime.timedelta(seconds=remain_time)
                    total_time = timedelta(seconds=int(total_time))
                    cost_time = datetime.timedelta(seconds=int(cost_time))

                    _PERCENT=f"{PERCENT: >6.2f}%"
                    SIGN, N1 = get_bar(percent)
                    _COST = f"{cost_time}|{total_time} "
                    _ETC = f"  ETC: {(datetime.datetime.now() + remain_time).strftime('%m-%d %H:%M:%S')}"

                    print('\r'+ Fore.CYAN +f"{_PERCENT}"+Fore.LIGHTBLACK_EX+SIGN+  \
                          Fore.LIGHTGREEN_EX +_COST+ Fore.LIGHTBLUE_EX +_ETC+ Fore.RESET, end='', flush=True)
                    # cprint(_PERCENT, fc=Fc.cyan)
                    # cprint(_COST, fc=Fc.green, coverage='')
                    # cprint(_ETC, fc=Fc.yellow, coverage='')
            if c == self.total_steps:
                print('\n')

            yield idx, i



if __name__=="__main__":
    def test_bar():
        for i in range(10):
            time.sleep(1)
            bar(i, 10)

    def test_probar():
        for idx, i in probar(range(15)):
            time.sleep(1)

    test_bar()


'''
This is a universal function library.
'''
from . import utils

__version__, __name__ = utils.version_config(update=False)
__version__ = '0.' + __version__
print('guang.__version__=:', __version__) # '0.0.7.2.7'
__author__ = 'K.y <beidongjiedeguang@gmail.com>'
__license__ = 'GPL-v3'
__copyright = 'Copyright 2020 K.y'

__all__ = ["ML","Utils","DL", "Voice", "wechat", "ML", "interesting", "app"]




import gdown
import fire
import numpy as np
import string

def rand_str(length=7):
    """Generate random string
    """
    totol_len = 62
    assert length < totol_len
    idx = np.random.permutation(totol_len)
    S = np.array([i for i in string.printable[:totol_len]])
    out = ''.join(S[idx[:length]].tolist())
    return out

def download(url, output=None, quiet=False, proxy=None, speed=None):
    if "https://drive.google.com/" in url:
        url_prefix = 'https://drive.google.com/uc?id='
        url_id = url[url.find('id=') + 3:]
        url = url_prefix + url_id
    gdown.download(url=url, output=output, quiet=quiet, proxy=proxy, speed=speed)

download.__doc__ = gdown.download.__doc__

if __name__ == "__main__":
    # url = "https://drive.google.com/open?id=1ZnDYF9rHKbef27tiHkWrLznqe18le7ol"
    # download(url, "xxx.zip", speed=100000) # speed: kb/s
    fire.Fire(download)


import plotly.graph_objects as go
import numpy as np
from guang.interesting import Lorenz
from guang.Utils.plotly import Scatter3d
from guang.Utils.animation.utils import ExtremeValue


def getRange(x, dx=0.02):
    delta = np.max(x) - np.min(x)
    delta = delta * dx
    return np.min(x) - delta, np.max(x) + delta

def lorenz_Traces(x0=1, y0=3, z0=5, lambd = 0.01,
                  k=3, t_start=0, t_end=10, all_step=900):

    t = np.linspace(t_start, t_end, all_step)
    Traces = []
    for i in range(k):
        trace = Lorenz.Trace(t=t, xyz=[x0+lambd*np.random.rand(),
                                       y0+lambd*np.random.rand(),
                                       z0+lambd*np.random.rand()])
        Traces.append(trace)
    return Traces

class Draw3D:
    def __init__(self):
        self.Traces = []
        self.initial_data = {}

    def get_extrem(self):
        Value = ExtremeValue()
        for trace in self.Traces:
            x,y,z = trace[:,0], trace[:,1], trace[:,2]
            xm, xM = getRange(x)
            ym, yM = getRange(y)
            zm, zM = getRange(z)
            xm, ym, zm = Value.get_min(x=xm, y=ym, z=zm)
            xM, yM, zM = Value.get_max(x=xM, y=yM, z=zM)
        return xm, xM, ym, yM, zm, zM

    def get_frames(self, total_frames = 100, period=1):
        """
        :param total_frames: total frames
        :param period: value interval:(0, 1] means the percentage of the total points.
        """
        self.k = len(self.Traces)
        N = len(self.Traces[0][:,0])
        step = N // total_frames
        i_start, i_end = 0, 0
        counts_period = int(N*period)
        frames = []
        for i in range(total_frames):
            i_end += step
            i_start = i_end - counts_period if i_end > counts_period else 0
            data = []

            for trace in self.Traces:
                x, y, z = trace[:, 0], trace[:, 1], trace[:, 2]
                data_x = x[i_start:i_end]
                data_y = y[i_start:i_end]
                data_z = z[i_start:i_end]
                data.append(go.Scatter3d(x=data_x, y=data_y, z=data_z,
                                         mode="lines+markers",
                                         line=dict(
                                             color=None,
                                             width=1.5),
                                         marker=dict(
                                             size=1.5,
                                             color=z,
                                             colorscale='Viridis')))

            if i ==0:
                self.initial_data.setdefault('data', data)
            frames.append(go.Frame(data=data,
                                   traces=[i for i in range(self.k)]  ####THIS IS THE LINE THAT MUST BE INSERTED
                                   ) )

        return frames


    def get_layout(self):
        xm, xM, ym, yM, zm, zM = self.get_extrem()
        layout = go.Layout(width=780, height=780,
                           scene={
                               'xaxis': {'range': [xm, xM], "nticks": 10, 'autorange': False, "zeroline": False},
                               # , 'rangemode': 'tozero', 'tickmode': "linear", 'tick0': -5, 'dtick': 1},
                               'yaxis': {'range': [ym, yM], "nticks": 10, 'autorange': False, "zeroline": False},
                               # , 'rangemode': 'tozero', 'tickmode': "linear", 'tick0': -5, 'dtick': 1},
                               'zaxis': {'range': [zm, zM], "nticks": 10, 'autorange': False, "zeroline": False},
                               # , 'rangemode': 'tozero', 'tickmode': "linear", 'tick0': -5, 'dtick': 1},
                               'aspectmode': 'cube',
                           },
                           title="Lorenz Curve",
                           hovermode="closest",
                           updatemenus=[dict(type="buttons",
                                             buttons=[dict(label="Play",
                                                           method="animate",
                                                           args=[None, dict(frame=dict(duration=20, redraw=True),
                                                                            transition=dict(duration=0),
                                                                            fromcurrent=True,
                                                                            mode='immediate'
                                                                            )])])])
        return layout

    def anima(self, total_frames=200, period=0.05):
        frames = self.get_frames(total_frames=total_frames, period=period)
        data = self.initial_data['data']
        fig = go.Figure(
            data=data,
            layout=self.get_layout(),
            frames=frames

        )
        return fig

class Draw2D:
    def __init__(self):
        self.Traces = []
        self.initial_data = {}

    def get_extrem(self):
        Value = ExtremeValue()
        for trace in self.Traces:
            x,y = trace[:,0], trace[:,1]
            xm, xM = getRange(x)
            ym, yM = getRange(y)

            xm, ym = Value.get_min(x=xm, y=ym)
            xM, yM = Value.get_max(x=xM, y=yM)
        return xm, xM, ym, yM

    def get_frames(self, total_frames = 100, period=1):
        """
        :param total_frames: total frames
        :param period: value interval:(0, 1] means the percentage of the total points.
        """
        self.k = len(self.Traces)

        N = len(self.Traces[0][:,0])
        step = N // total_frames
        i_start, i_end = 0, 0
        counts_period = int(N*period)
        frames = []
        for i in range(total_frames):
            i_end += step
            i_start = i_end - counts_period if i_end > counts_period else 0
            data = []

            for trace in self.Traces:
                x, y = trace[:, 0], trace[:, 1]
                data_x = x[i_start:i_end]
                data_y = y[i_start:i_end]
                data.append(go.Scatter(
                    x=data_x,
                    y=data_y,
                    mode="lines",
                    line=dict(color=None, width=2)))

            if i ==0:
                self.initial_data.setdefault('data', data)
            frames.append(go.Frame(data=data,
                                   traces=[i for i in range(self.k)]  #THIS IS THE LINE THAT MUST BE INSERTED
                                   ) )
        return frames

    def get_layout(self, width=980,height=880, duration=10,
                   title="Lorenz X-Y 2D Curve",
                   redraw=False,
                   mode='immediate'):
        xm, xM, ym, yM= self.get_extrem()

        # height = int(width* (yM-ym)/(xM-xm)) + 100
        layout = go.Layout(width=width, height=height,
                           xaxis=dict(range=[xm, xM], autorange=False, zeroline=False),
                           yaxis=dict(range=[ym, yM], autorange=False, zeroline=False),
                           title=title,
                           hovermode="closest",
                           updatemenus=[dict(type="buttons",
                                             buttons=[dict(label="Play",
                                                           method="animate",
                                                           args=[None,
                                                                 dict(frame=dict(duration=duration, redraw=redraw),
                                                                      transition=dict(duration=0), fromcurrent=True,
                                                                      mode=mode)])])])
        return layout


    def anima(self, total_frames=200, period=0.05, width=990, height=900):
        frames = self.get_frames(total_frames=total_frames, period=period)
        data = self.initial_data['data']
        layout = self.get_layout(width=width, height=height)
        fig = go.Figure(
            data=data,
            layout=layout,
            frames=frames

        )
        return fig

def Traces2d(r=10, all_step=1000):

    theta = np.linspace(0, 2*np.pi, all_step)[...,None]
    x = r * np.cos(theta)
    y = r * np.sin(theta)
    Trace = []
    Trace.append(np.concatenate((x, y), axis=1))
    Trace.append(np.concatenate((theta+r, x), axis=1))
    Trace.append(np.concatenate((theta-r, y), axis=1))
    return Trace

if __name__ == "__main__":
    draw2d = Draw2D()
    draw2d.Traces = Traces2d(r=10)
    fig = draw2d.anima(total_frames=200, period=1)
    fig.show()

    # draw3d = Draw3D()
    # draw3d.Traces = lorenz_Traces(x0=5, y0=6, z0=10, lambd=10,
    #                               k=5, all_step=900)
    # ---------------------------------------------------- option start
    # xm, xM, ym, yM, zm, zM = draw3d.get_extrem()
    # draw3d.initial_data =  [go.Scatter3d(x=[xm], y=[ym], z=[zm]),
    #                     go.Scatter3d(x=[xM], y=[yM], z=[zM]),
    #                     go.Scatter3d(x=[xm], y=[yM], z=[zm]), ]
    # ---------------------------------------------------- option end

    # fig = draw3d.anima(total_frames=200, period=0.05)
    # fig.show()




from numpy import *
import matplotlib.pyplot as plt
def get_functions(x):
    operators = ['+','-','*','/','**','%','^']
    functions = ['sin','cos','exp','log','1.*']
    constants = ['pi','e']
    variate=  ['x']
    c = random.choice(constants)
    o = random.choice(operators)
    f = random.choice(functions)
    v = random.choice(variate)
    
    out = c + o + f + '(' + c + o + v+')'
    return eval(out), out

if __name__ == '__main__':
    x = linspace(0, 2*pi, 100)
    y, sy = get_functions(x=x)
    plt.plot(x, y)
    plt.show()
    


class ExtremeValue:
    """Find Extreme Value
    Example:
    ```
    Value = ExtremeValue()
    for x, y in zip(X, Y):
        x_min, y_min = Value.get_min(x=x, y=y)
        x_max, y_max = Value.get_max(x=x, y=y)
    ```
    """
    def __init__(self):
        self.x_min = {}
        self.x_max = {}

    def get_min(self, **x):
        res = []
        for key in x:
            self.x_min.setdefault(key, np.inf)
            if x[key] < self.x_min[key]:
                self.x_min[key] = x[key]
            res.append(self.x_min[key])
        if len(res) > 1:
            return res
        else:
            return res[0]

    def get_max(self, **x):
        res = []
        for key in x:
            self.x_max.setdefault(key, -np.inf)
            if x[key] > self.x_max[key]:
                self.x_max[key] = x[key]
            res.append(self.x_max[key])
        if len(res) > 1:
            return res
        else:
            return res[0]


import os
import pickle

def version_config(update=False):
    filename = os.path.join(os.path.dirname(__file__), 'version_config.yaml')
    config = yaml_load(filename)
    # config = load(filename)
    if update:
        config["version"] += 1
        yaml_dump(filename, config)
        # save(filename, config)
    __version__ = '0.' + '.'.join(list(str(config["version"])))
    __name__ = config["name"]
    return __version__, __name__

def save(filename, file):
    with open(filename, 'wb') as fw:
        pickle.dump(file, fw)

def load(filename):
    with open(filename, 'rb') as fi:
        file = pickle.load(fi)
    return file

def yaml_dump(filepath, data):
    from yaml import dump
    try:
        from yaml import CDumper as Dumper
    except ImportError:
        from yaml import Dumper
    with open(filepath, "w", encoding='utf-8') as fw:
        fw.write(dump(data, Dumper=Dumper))
        
def yaml_load(filepath):
    from yaml import load
    try:
        from yaml import CLoader as Loader
    except ImportError:
        from yaml import Loader
    with open(filepath, 'r', encoding="utf-8") as stream:
    #     stream = stream.read()
        content = load(stream, Loader=Loader)
    return content




import argparse
def main():
    parser = argparse.ArgumentParser()
    parser.description = 'give me two number, out you one number!'
    parser.add_argument('-a',help='number 1', type=int)
    parser.add_argument('-b',help='number 2', type=int)
    args = parser.parse_args()
    a = args.a
    b = args.b
    print(f'emmm, I give you this:{(a+b)/2}')

if __name__ == '__main__':
    main()



import pickle
import itchat
from itchat.content import *
import time
from glob import glob
import os
from guang.Utils.toolsFunc import path
from datetime import datetime
from multiprocessing import Process
from multiprocessing import Pool
import threading
from concurrent.futures import ThreadPoolExecutor
from guang.Utils.symbols import fstring
from guang.Utils.toolsFunc import rm
import mdmail
import psutil

def get_all_info():
    """get all my friends' information """
    nickNames = []

    for friend in itchat.get_friends():
        nickNames.append(friend.NickName)
        # print(nickNames)
    return nickNames


def get_userName(*name_list):
    UserNames = {}
    for name in name_list:
        friends = itchat.search_friends(name=name)

        if len(friends) == 1:
            UserNames[name] = friends[0].UserName
        else:
            print(f'name: “{name}” is NOT unique')
            raise Exception('')
    return UserNames


def save_userName():
    name_list = ['爸爸', '妈妈',
                 '被冻结的光', 'caloi']
    userNames = get_userName(*name_list)
    with open('userNames.pkl', 'wb') as fo:
        pickle.dump(userNames, fo)


def load_userName():
    with open('userNames.pkl', 'rb') as fi:
        userNames = pickle.load(fi)


def get_list(lis=[''], count=[0]):
    '''get msg list and count'''

    @itchat.msg_register([TEXT, MAP, CARD, NOTE, SHARING, PICTURE, RECORDING, ATTACHMENT, VIDEO])
    def reply(msg):
        lis[0] = msg
        count[0] = count[0] + 1
        return

    return lis[0], count[0]


count_n = 0
def get_content(l, count):
    global count_n
    if count == count_n:
        count_n += 1
        return l
    else:
        return False


def get_msg():
    #     global = count_n
    l, count = get_list()
    message = get_content(l, count)
    if message:
        return message
    else:
        return None


def dynamic_msg():
    '''Dynamic acquisition of anyone's message
    Once acquired, the program releases'''
    while 1:
        msg = get_msg()
        if msg == None:
            time.sleep(.6)
            itchat.configured_reply()
            continue
        else:
            return msg


def dynamic_specified_msg(userName=None):
    '''
    Dynamic acquisition of specified user's message
    Once acquired, the program releases
    '''
    while 1:
        msg = get_msg()
        try:
            msg_userName = msg.user.userName
        except:
            msg_userName = None

        if msg_userName == None or msg_userName != userName:
            time.sleep(.6)
            itchat.configured_reply()
            continue
        else:
            return msg


flag_get_txt = True
MY_USER_NAME = None
def _txt(msg, MyName="被冻结的光"):
    global flag_get_txt, MY_USER_NAME
    if flag_get_txt:
        MY_USER_NAME = get_userName(MyName)[MyName]
        # print(MY_USER_NAME)
        flag_get_txt = False
    tpath = os.path.join('wechat_download', msg.User.NickName, 'text')
    if not os.path.exists(tpath):
        os.makedirs(tpath)
    nowTime = datetime.now()
    nowTime = nowTime.strftime("%Y-%m-%d %H:%M:%S")
    out_text = fstring(msg.text, 100, '<', flag='en')
    if msg['FromUserName'] != MY_USER_NAME:
        out_name = fstring(msg.User.NickName, 10, '<')
    else:
        out_name = fstring(MyName, 10, '<')
    text = f"{out_name}   :{out_text}{nowTime: >10}\n"
    with open(path(tpath + '/dialogue.txt'), 'a+', encoding='utf-8') as fw:
        fw.write(text)
    return msg

def write_txt(name_list, MyName="被冻结的光", fileDir="itchat.pkl", cmdQR=False):
    """name_list such as ['filehelper', 'caloi']"""
    lock = threading.Lock()
    def relogin(fileDir):
        # global lock
        sleepTime = 1200
        while (True):
            lock.acquire()
            print("lock acquire")
            if os.path.exists(fileDir) and time.time() - os.path.getatime(fileDir) > 2*sleepTime:
                rm(fileDir)
            print("................loading....................")
            if itchat.load_login_status(fileDir):
                pass
            else:
                if cmdQR:
                    itchat.login(enableCmdQR=2)
                else:
                    picDir = 'QR.png'
                    itchat.login(picDir=picDir)
                    # to modified: miniconda3/lib/python3.7/site-packages/itchat/utils.py
            itchat.dump_login_status(fileDir=fileDir)
            print("................load success ................")
            print("lock release")
            lock.release()
            time.sleep(sleepTime)

            pid = os.getpid()
            p = psutil.Process(pid)
            p.kill()


    def run(nickName):
        # global lock
        print(nickName, ":start")
        @itchat.msg_register([TEXT])
        def write2file(msg):
            userName = get_userName(nickName)[nickName]
            if msg.user.userName == userName:
                _txt(msg,MyName)
        lock.acquire()
        # itchat.auto_login(hotReload=True, statusStorageDir=fileDir, enableCmdQR=2)
        itchat.run(debug=True)

    try:
        #  ----------------------------------------------------------------------
        # t0 = threading.Thread(target=relogin, args=(fileDir))
        # t = [threading.Thread(target=run, args=(name,)) for name in name_list]
        # t0.start()
        # [i.start() for i in t]

        # ----------------------------------------------------------------------
        # with Pool(len(name_list)) as pool:  # process pool
        #     pool.map(run, name_list)
        # ----------------------------------------------------------------------
        # ----------------------------------------------------------------------
        # Thread pools without locks can result in multiple writes
        with ThreadPoolExecutor(max_workers=len(name_list) + 5) as executor:
            res0 = executor.submit(relogin, fileDir)
            res = [executor.submit(run, name) for name in name_list]

    except:
        print("try is not successful !")

def download_file(msg, fileType='mp3'):
    '''
    Param
    -----
        fileType : 'attachment' ,'mp3', 'mp4', 'png'
        and ATTACHMENT,  of which include various file types.
    Returns
    -------
        msg, file_path
    '''
    #     msg = dynamic_specified_msg(username)
    file_name = msg.fileName.split('.')
    sufname = file_name[-1]
    prename = '.'.join(file_name[:-1])

    if fileType == 'attachment':  # 'attachment':
        tpath = os.path.join('wechat_download', msg.User.NickName, 'attachment')
        if not os.path.exists(tpath):
            os.makedirs(tpath)
        msg.download(os.path.join(tpath, msg.fileName))
    elif sufname == 'png' and fileType == 'png':
        tpath = os.path.join('wechat_download', msg.User.NickName, 'picture')
        if not os.path.exists(tpath):
            os.makedirs(tpath)
        msg.download(os.path.join(tpath, msg.fileName))
    elif sufname == 'mp3' and fileType == 'mp3':
        tpath = os.path.join('wechat_download', msg.User.NickName, 'voice')
        if not os.path.exists(tpath):
            os.makedirs(tpath)
        msg.download(os.path.join(tpath, msg.fileName))
    elif sufname == 'mp4' and fileType == 'mp4':
        tpath = os.path.join('wechat_download', msg.User.NickName, 'video')
        if not os.path.exists(tpath):
            os.makedirs(tpath)
        msg.download(os.path.join(tpath, msg.fileName))

    #     os.remove(msg.fileName) # remove 当前文件夹下的下载文件
    return msg, tpath


def d_time(d_t, t0=[]):
    """
    The unit of dt is seconds.
    Returns 1 when current time in `d_t` duration
    Returns 0 when current time out duration
    """
    if not t0:
        t0.append(time.time())
    t1 = time.time()
    if t1 - t0[0] < d_t:
        return True
    else:
        return False



if __name__ == '__main__':
    statusDir = "itchat.pkl"
    name_list = ['妈妈','caloi']
    while True:
        t_i = Process(target=write_txt, args=(name_list, "被冻结的光", statusDir, False) )
        t_i.start()
        t_i.join()



    # itchat.auto_login(hotReload=True)
    # while d_time(60):
    #     msg = dynamic_specified_msg(get_userName('光')['光'])
    #     msg = download_file(msg, fileType='mp3')





import string
import re 

useless = string.printable + "，。；’：‘’“”！？《》￥…（）—·【】、"


def has_chinese(str0):
    """判断是否存在汉字
    """
    return bool(re.search(u'[\u4e00-\u9fa5]', str0))
def has_english(str0):
    return bool(re.search('[a-z]', str0.lower()))

def has_number(str0):
    return bool(re.search('[0-9]', str0))

def replace_bracket(a):
    l0 = a.find('（')
    l1 = a.find('）')
    a=a.replace(a[l0:l1+1],'')
    return a

def sub2empty(useless, str0):
    return re.sub(f"[{useless}]+", "", str0)

def fstring(string, N=20, align='>', symbol=None, flag='en'):
    """Chinese and English mixed string alignment
     chr(12288): Chinese space"""
    if flag == 'zh':
        space = chr(12288) # chinese space
    elif flag == 'en':
        space = chr(32) # english space
    def lenStr(string, space): # if space is chr(12288)
        if space == chr(12288):
            zh, en = 1, 0.6
        elif space == ' ':
            zh, en = 1.83, 1
        count = 0
        for i in string:
            if has_chinese(i):
                count += zh
            else:
                count += en
        return count
    n = round(lenStr(string, space))
    if symbol == None:
        symbol = space
    if align == '>':
        out = (N-n)*symbol+string
    elif align == "<":
        out = string + symbol*(N-n)
    elif align == "^":
        left = (N-n)//2
        right = N-n-left
        out = left* symbol + string + right* symbol
    else:
        "align options: > < ^"
    return out


if __name__ == "__main__":
    # print(useless)
    a = """阿斯顿发生啊空手ssssssdfasd道解为了阿萨；爱哭的时间斯京东方"""
    b = """绿卡就是asdfasdf咔叽"""
    print(fstring(a, 100, '>', flag='en'))
    print(fstring(b, 100, '>', flag='en'))
    print(fstring(a, 50, '>', flag='zh'))
    print(fstring(b, 50, '>', flag='zh'))


# tensorflow2.0
import tensorflow as tf
from tensorflow.keras import Input,Model, layers, regularizers 
from tensorflow.keras.layers import Dense
import numpy as np

class Encoder(tf.keras.Model):
    def __init__(self, w, h, dim=256):
        super().__init__()
        self.fc1 = Dense(dim, activation='relu', input_shape=(w*h,) )                         
        self.fc2 = Dense(dim/2, activation='relu')
        self.fc3 = Dense(dim/4, activation='relu')
    def call(self,x):
        x = self.fc1(x)
        x = self.fc2(x)
        x = self.fc3(x)
        return x

class Decoder(tf.keras.Model):
    def __init__(self, w, h, dim=256):
        super().__init__()
        Out_dim = w*h

        self.fc1 = Dense(dim/2, activation='relu', input_shape=(dim,) )
        self.fc2 = Dense(dim, activation='relu')
        self.fc3 = Dense(Out_dim, activation='sigmoid')
        
    def call(self,x):
        x = self.fc1(x)
        x = self.fc2(x)
        x = self.fc3(x)
        return x

class DAE:
	def __init__(self, dim, w, h):
		self.dim = dim
		self.w, self.h = w, h
	def compile(self):
		input_img = Input(shape=(self.w*self.h),)
		self.encoder = Encoder(self.w,self.h, self.dim)
		self.encoded = self.encoder(input_img)

		self.decoder = Decoder(self.w,self.h, self.dim)
		self.decoded = self.decoder(self.encoded)

		self.autoencoder = Model(input_img, self.decoded)
		self.autoencoder(input_img)
		self.autoencoder.compile(optimizer=tf.keras.optimizers.Adam(), loss='binary_crossentropy')
	def fit(self, x_train,x_test, epochs=10, batch_size=256):
		self.autoencoder.fit(x_train, x_train,
						epochs=epochs,
						batch_size=batch_size,
						shuffle=True,
						validation_data=(x_test, x_test))

def TEST():
	from tensorflow.keras.datasets import mnist
	import matplotlib.pyplot as plt
	(x_train, _), (x_test, _) = mnist.load_data()
	x_train = x_train.astype('float32') / 255.
	x_test = x_test.astype('float32') / 255.
	x_train = x_train.reshape((len(x_train), np.prod(x_train.shape[1:])))
	x_test = x_test.reshape((len(x_test), np.prod(x_test.shape[1:])))
	dim = 256
	w,h = 28,28
	ae = DAE(dim, w, h)
	ae.compile()
	ae.fit(x_train, x_test)

	decoded_imgs = ae.autoencoder.predict(x_test)
	n = 10  # how many digits we will display
	plt.figure(figsize=(20, 4))
	for i in range(n):
		# display original
		ax = plt.subplot(2, n, i + 1)
		loc = i + 20
		plt.imshow(x_test[loc].reshape(28, 28))
		plt.gray()
		ax.get_xaxis().set_visible(False)
		ax.get_yaxis().set_visible(False)

		# display reconstruction
		ax = plt.subplot(2, n, i + 1 + n)
		plt.imshow(decoded_imgs[loc].reshape(28, 28))
		plt.gray()
		ax.get_xaxis().set_visible(False)
		ax.get_yaxis().set_visible(False)
	plt.show()
if __name__ == "__main__":
	TEST()


from datetime import datetime, timedelta
from itertools import accumulate
import requests
import dill
import os

def download_data():
    pwd_path = os.path.dirname(__file__)
    data_path = os.path.join(pwd_path, 'date_data')
    if os.path.exists(data_path):
        pass
    else:
        print('Downloading the required file')
        res = requests.get('https://raw.githubusercontent.com/beidongjiedeguang/guang/master/guang/Utils/date_data')
        with open(data_path, 'wb') as fo:
            fo.write(res.content)
    return data_path
data_path = download_data()
with open(data_path, 'rb') as fi:
    [CHINESENEWYEAR,CHINESEYEARCODE] = dill.load(fi)
# The class `LunarDate` is modified from https://github.com/CutePandaSh/zhdate.git
class LunarDate():

    def __init__(self, lunar_year, lunar_month, lunar_day, leap_month=False):
        """初始化函数

        Arguments:
            lunar_year {int} -- 农历年
            lunar_month {int} -- 农历月份
            lunar_day {int} -- 农历日

        Keyword Arguments:
            leap_month {bool} -- 是否是在农历闰月中 (default: {False})

        """
        if not LunarDate.validate(lunar_year, lunar_month, lunar_day, leap_month):
            raise TypeError('农历日期不支持，超出农历1900年1月1日至2100年12月29日，或日期不存在')
        self.lunar_year = lunar_year
        self.lunar_month = lunar_month
        self.lunar_day = lunar_day
        self.leap_month = leap_month
        self.year_code = CHINESEYEARCODE[self.lunar_year - 1900]
        self.newyear = datetime.strptime(CHINESENEWYEAR[self.lunar_year - 1900], '%Y%m%d')

    def to_datetime(self):
        """农历日期转换称公历日期

        Returns:
            datetime -- 当前农历对应的公历日期
        """
        return self.newyear + timedelta(days=self.__days_passed())

    @staticmethod
    def from_datetime(dt):
        """静态方法，从公历日期生成农历日期

        Arguments:
            dt {datetime} -- 公历的日期

        Returns:
            LunarDate -- 生成的农历日期对象
        """
        lunar_year = dt.year
        if (datetime.strptime(CHINESENEWYEAR[lunar_year - 1900], '%Y%m%d') - dt).days > 0:
            lunar_year -= 1
        newyear_dt = datetime.strptime(CHINESENEWYEAR[lunar_year - 1900], '%Y%m%d')
        days_passed = (dt - newyear_dt).days
        year_code = CHINESEYEARCODE[lunar_year - 1900]
        month_days = LunarDate.decode(year_code)

        for pos, days in enumerate(accumulate(month_days)):
            if days_passed + 1 <= days:
                month = pos + 1
                lunar_day = month_days[pos] - (days - days_passed) + 1
                break

        leap_month = False
        if (year_code & 0xf) == 0 or month <= (year_code & 0xf):
            lunar_month = month
        else:
            lunar_month = month - 1

        if (year_code & 0xf) != 0 and month == (year_code & 0xf) + 1:
            leap_month = True

        return LunarDate(lunar_year, lunar_month, lunar_day, leap_month)

    @staticmethod
    def today():
        return LunarDate.from_datetime(datetime.now())

    def __days_passed(self):
        """私有方法，计算当前农历日期和当年农历新年之间的天数差值

        Returns:
            int -- 差值天数
        """
        month_days = LunarDate.decode(self.year_code)
        month_leap = self.year_code & 0xf  # 当前农历年的闰月，为0表示无闰月

        if (month_leap == 0) or (self.lunar_month < month_leap):  # 当年无闰月，或者有闰月但是当前月小于闰月
            days_passed_month = sum(month_days[:self.lunar_month - 1])
        elif (not self.leap_month) and (self.lunar_month == month_leap):  # 当前不是闰月，并且当前月份和闰月相同
            days_passed_month = sum(month_days[:self.lunar_month - 1])
        else:
            days_passed_month = sum(month_days[:self.lunar_month])

        return days_passed_month + self.lunar_day - 1

    def chinese(self):
        ZHNUMS = '零一二三四五六七八九十'
        zh_year = ''
        for i in range(0, 4):
            zh_year += ZHNUMS[int(str(self.lunar_year)[i])]
        zh_year += '年'

        if self.leap_month:
            zh_month = '闰'
        else:
            zh_month = ''

        if self.lunar_month == 1:
            zh_month += '正'
        elif self.lunar_month == 12:
            zh_month += '腊'
        elif self.lunar_month <= 10:
            zh_month += ZHNUMS[self.lunar_month]
        else:
            zh_month += f"十{ZHNUMS[self.lunar_month - 10]}"
        zh_month += '月'

        if self.lunar_day <= 10:
            zh_day = f'初{ZHNUMS[self.lunar_day]}'
        elif self.lunar_day < 20:
            zh_day = f'十{ZHNUMS[self.lunar_day - 10]}'
        elif self.lunar_day == 20:
            zh_day = '二十'
        elif self.lunar_day < 30:
            zh_day = f'二十{ZHNUMS[self.lunar_day - 20]}'
        else:
            zh_day = '三十'

        year_tiandi = LunarDate.__tiandi(self.lunar_year - 1900 + 36) + '年'

        shengxiao = "鼠牛虎兔龙蛇马羊猴鸡狗猪"

        return f"{zh_year}{zh_month}{zh_day} {year_tiandi} ({shengxiao[(self.lunar_year - 1900) % 12]}年)"

    def __str__(self):
        """打印字符串的方法

        Returns:
            str -- 标准格式农历日期字符串
        """
        if self.leap_month:
            return f"农历{self.lunar_year}年闰{self.lunar_month}月{self.lunar_day}日"
        else:
            return f"农历{self.lunar_year}年{self.lunar_month}月{self.lunar_day}日"

    def __repr__(self):
        return self.__str__()

    def __eq__(self, another):
        if not isinstance(another, LunarDate):
            raise TypeError('比较必须都是LunarDate类型')
        cond1 = self.lunar_year == another.lunar_year
        cond2 = self.lunar_month == another.lunar_month
        cond3 = self.lunar_day == another.lunar_day
        cond4 = self.leap_month == another.leap_month
        return cond1 and cond2 and cond3 and cond4

    def __add__(self, another):
        if not isinstance(another, int):
            raise TypeError('加法只支持整数天数相加')
        return LunarDate.from_datetime(self.to_datetime() + timedelta(days=another))

    def __sub__(self, another):
        if isinstance(another, int):
            return LunarDate.from_datetime(self.to_datetime() - timedelta(days=another))
        elif isinstance(another, LunarDate):
            return (self.to_datetime() - another.to_datetime()).days
        elif isinstance(another, datetime):
            return (self.to_datetime() - another).days
        else:
            raise TypeError('减法只支持整数，LunarDate, Datetime类型')


    @staticmethod
    def __tiandi(anum):
        tian = '甲乙丙丁戊己庚辛壬癸'
        di = '子丑寅卯辰巳午未申酉戌亥'
        return f'{tian[anum % 10]}{di[anum % 12]}'

    @staticmethod
    def validate(year, month, day, leap):
        """农历日期校验

        Arguments:
            year {int} -- 农历年份
            month {int} -- 农历月份
            day {int} -- 农历日期
            leap {bool} -- 农历是否为闰月日期

        Returns:
            bool -- 校验是否通过
        """
        # 年份低于1900，大于2100，或者月份不属于 1-12，或者日期不属于 1-30，返回校验失败
        if not (1900 <= year <= 2100 and 1 <= month <= 12 and 1 <= day <= 30):
            return False

        year_code = CHINESEYEARCODE[year - 1900]

        # 有闰月标志
        if leap:
            if (year_code & 0xf) != month:  # 年度闰月和校验闰月不一致的话，返回校验失败
                return False
            elif day == 30:  # 如果日期是30的话，直接返回年度代码首位是否为1，即闰月是否为大月
                return (year_code >> 16) == 1
            else:  # 年度闰月和当前月份相同，日期不为30的情况，返回通过
                return True
        elif day <= 29:  # 非闰月，并且日期小于等于29，返回通过
            return True
        else:  # 非闰月日期为30，返回年度代码中的月份位是否为1，即是否为大月
            return ((year_code >> (12 - month) + 4) & 1) == 1

    @staticmethod
    def decode(year_code):
        """解析年度农历代码函数

        Arguments:
            year_code {int} -- 从年度代码数组中获取的代码整数

        Returns:
            [int] -- 当前年度代码解析以后形成的每月天数数组，已将闰月嵌入对应位置，即有闰月的年份返回长度为13，否则为12
        """
        month_days = list()
        for i in range(5, 17):
            if (year_code >> (i - 1)) & 1:
                month_days.insert(0, 30)
            else:
                month_days.insert(0, 29)
        if year_code & 0xf:
            if year_code >> 16:
                month_days.insert((year_code & 0xf), 30)
            else:
                month_days.insert((year_code & 0xf), 29)
        return month_days

    @staticmethod
    def month_days(year):
        """根据年份返回当前农历月份天数list

        Arguments:
            year {int} -- 1900到2100的之间的整数

        Returns:
            [int] -- 农历年份所对应的农历月份天数列表
        """
        return LunarDate.decode(
            CHINESEYEARCODE[year - 1900]
        )


    
"""the following code is copied from the network"""

import math
import ephem

def chinaCalendar(date): # 默认输入ut+8时间
    lunar_yue =["正月","二月","三月","四月","五月","六月","七月","八月","九月","十月","十一月","十二月"]
    nlrq = ["初一","初二","初三","初四","初五","初六","初七","初八","初九","初十","十一","十二","十三","十四","十五","十六","十七","十八","十九","二十","廿一","廿二","廿三","廿四","廿五","廿六","廿七","廿八","廿九","三十"]
    tiangan = ["甲","乙","丙","丁","戊","己","庚","辛","壬","癸"]
    dizhi = ["子","丑","寅","卯","辰","巳","午","未","申","酉","戌","亥"]
    gz = [''] * 60  # 六十甲子表
    for i in range(60):
        gz[i] = tiangan[i % 10] + dizhi[i % 12]

    def EquinoxSolsticeJD(year, angle):
        if 0 <= angle < 90:
            date = ephem.next_vernal_equinox(year)
        elif 90 <= angle < 180:
            date = ephem.next_summer_solstice(year)
        elif 180 <= angle < 270:
            date = ephem.next_autumn_equinox(year)
        else:
            date = ephem.next_winter_solstice(year)
        JD = ephem.julian_date(date)
        return JD

    # 计算二十四节气
    def SolarLongitube(JD):
        date = ephem.Date(JD - 2415020)
        s = ephem.Sun(date)  # date应为UT时间
        sa = ephem.Equatorial(s.ra, s.dec, epoch=date)
        se = ephem.Ecliptic(sa)
        L = se.lon / ephem.degree / 180 * math.pi
        return L

    def SolarTerms(year, angle):
        if angle > 270: year -= 1
        if year == 0: year -= 1  # 公元0改为公元前1
        JD = EquinoxSolsticeJD(str(year), angle)  # 初值
        JD1 = JD
        while True:
            JD2 = JD1
            L = SolarLongitube(JD2)
            JD1 += math.sin(angle * math.pi / 180 - L) / math.pi * 180
            if abs(JD1 - JD2) < 0.00001:
                break # 精度小于1 second
        return JD1 # UT

    def DateCompare(JD1, JD2): # 输入ut，返回ut+8的比较结果
        JD1 += 0.5 + 8/24
        JD2 += 0.5 + 8/24
        if int(JD1) >= int(JD2): return True
        else: return False

    def dzs_search(year): # 寻找年前冬至月朔日
        if year == 1: year -= 1  # 公元0改为公元前1
        dz = ephem.next_solstice(str(year-1) + '/12') # 年前冬至
        jd = ephem.julian_date(dz)
        # 可能的三种朔日
        date1 = ephem.next_new_moon(ephem.Date(jd - 2415020 - 0))
        jd1 = ephem.julian_date(date1)
        date2 = ephem.next_new_moon(ephem.Date(jd - 2415020 - 29))
        jd2 = ephem.julian_date(date2)
        date3 = ephem.next_new_moon(ephem.Date(jd - 2415020 - 31))
        jd3 = ephem.julian_date(date3)
        if DateCompare(jd, jd1): # 冬至合朔在同一日或下月
            return date1
        elif DateCompare(jd, jd2) and (not DateCompare(jd, jd1)):
            return date2
        elif DateCompare(jd, jd3): # 冬至在上月
            return date3
    
    
    JD = ephem.julian_date(date) - 8/24 # ut
    year = ephem.Date(JD + 8/24 - 2415020).triple()[0]
    shuo = []
    shuo.append(dzs_search(year))  # 冬至朔
    sJD1 = ephem.julian_date(shuo[0])
    next_dzs = dzs_search(year+1) # 次年冬至朔
    dzsJD = ephem.julian_date(next_dzs)
    if DateCompare(JD, dzsJD):
        shuo[0] = next_dzs
        next_dzs = dzs_search(year+2)
        dzsJD = ephem.julian_date(next_dzs)
    # 查找所在月及判断置闰
    run = ''
    szy = 0
    j = -1 # 计算连续两个冬至月中的合朔次数
    zry = 99 # 无效值
    flag = False
    while not DateCompare(sJD1, dzsJD): # 次年冬至朔前合朔即当年月份
        j += 1
        sJD1 = ephem.julian_date(shuo[j]) # 起冬至朔
        if DateCompare(JD, sJD1):
            szy += 1 # date所在月
            newmoon = int(sJD1 + 8/24 + 0.5)
        shuo.append(ephem.next_new_moon(shuo[j])) # i+1月朔
        if j == 0: continue # 冬至月一定含中气，从次月开始查找
        sJD2 = ephem.julian_date(shuo[j+1])  # 次月朔
        month = ephem.Date(sJD1 - 2415020).triple()[1] # 求某月的中气
        angle = (-90 + 30 * month) % 360 # 该月中气
        if j == 1:
            nian1 = ephem.Date(sJD1 - 2415020).triple()[0]
            qJD1 = SolarTerms(nian1, angle) # 每月中气
        else:
            qJD1 = qJD2 # 使用上次计算结果，节约计算
        nian2 = ephem.Date(sJD2 - 2415020).triple()[0]
        qJD2 = SolarTerms(nian2, (angle+30)%360) # 次月中气
        if not DateCompare(qJD1, sJD1) and DateCompare(qJD2, sJD2) and flag == False:
                zry = j + 1 # 置闰月
                flag = True # 仅第一个无中气月置闰
    rq = int(JD + 8/24 + 0.5) - newmoon # 日干支
    if j == 12 and zry != 99: # 连续两个冬至月间合朔12次则不闰
        zry = 99
    if szy % 12 == zry % 12 and zry != 99:
        run = '闰'
    if szy >= zry % 12 and zry != 99:
        szy -= 1
    # 判断节气月
    month = ephem.Date(date).triple()[1]
    angle = (-135 + 30 * month) % 360
    jJD2 = SolarTerms(year, (angle+30)%360) # 次月节气
    if angle == 225: # 再次月节气
        jJD3 = SolarTerms(year+1, (angle+60)%360)
    else:
        jJD3 = SolarTerms(year, (angle + 60) % 360)
    if angle == 255: # 每月节气
        jJD1 = SolarTerms(year-1, angle)
    else:
        jJD1 = SolarTerms(year, angle)
    daxue = False
    if DateCompare(JD, jJD3): # JD ≥ JD3
        jq = ephem.Date(jJD3 + 8/24 - 2415020)
        month2 = (angle + 15 - 210) // 30 % 12
        if angle == 225: daxue = True
    elif DateCompare(JD, jJD2): # JD2 ≤ JD ＜ JD3
        jq = ephem.Date(jJD2 + 8/24 - 2415020)
        month2 = (angle + 15 - 240) // 30 % 12
        if angle == 225 or (jq.triple()[1] == 12 and angle == 255): daxue = True  # 大雪或小寒
    else: # JD ＜ JD2
        jq = ephem.Date(jJD1 + 8/24 - 2415020)
        month2 = (angle + 15 - 270) // 30 % 12
        if angle == 255: daxue = True
    nian2 = jq.triple()[0]
    if nian2 < 0: nian2 += 1
    if daxue == True: nian2 += 1
    jqy = gz[(nian2 * 12 + month2 + 12) % 60] # 月干支
    if (szy - 3) % 12 >= 10 and ephem.Date(date).triple()[1] <= 3: year -= 1 # 年干支
    if year < 0: year += 1
        
    lunar_year = year
    lunar_month = (szy - 3) % 12 + 1
    lunar_day = rq + 1
    
    chinese_year = gz[(year - 4) % 60] + '年'
    chinese_month = jqy + '月'
    chinese_day = gz[int(JD + 8/24 + 0.5 + 49) % 60] + '日'
    return lunar_year, lunar_month, lunar_day, [chinese_year, chinese_month, chinese_day]


import numpy as np
import distance as Dist

def is_similarity(vec1, vec2, dist_choise=2, tolerance=0.3):
    '''
    input
    -----
        dist_choise: 选择哪种距离, 1:P1范数(曼哈顿距离), 2: P2范数(欧氏距离), 3: P_inf范数(切比雪夫距离)
        tolerance: 相似度小于该值时会被判断为同类别
    return
    ------
        bool
    '''
    if dist_choise == 1:
        distance = Dist.manhattan_dist
    elif dist_choise == 2:
        distance = Dist.euclidean_dist
    elif dist_choise == 3:
        distance = Dist.chebyshev_dist
    else:
        raise 'dist_choise is a bad number'
        
    # 判断相似度:
    if distance(vec1, vec2) < tolerance:
        return True
    else:
        return False

def find_simil_idx(Vec_list, VEC,dist_choise=2, tolerance=0.3):

    IDX = []
    for idx, vec in enumerate(Vec_list):
        if is_similarity(vec, VEC,dist_choise, tolerance):
            IDX.append(idx)
    return IDX




def simil_score(V1, w1, V2, w2, dist_choise=2, lamb=5, sigma=1):
    '''
    V1,w1对应Key color
    V2,w2对应待分类color
    可调节参数:
    lamb: 取值建议1~10,值越大表示颜色百分比所占权重越大
    sigma: 取值建议0~1.7, 值越小表示相似距离变大(小)时，相似度权重下降(上升)越快, 它的取值与 dist_choise相关:P1(0~3),P2(0~1.7),P3(0~1)
    output: 0~1之间的相似分数
    '''
    if dist_choise == 1:
        distance = Dist.manhattan_dist
    elif dist_choise == 2:
        distance = Dist.euclidean_dist
    elif dist_choise == 3:
        distance = Dist.chebyshev_dist
    else:
        raise 'dist_choise is a bad number'
    dis = 0
    Dis = 0
    for i1, v1 in enumerate(V1):
        for i2, v2 in enumerate(V2):
            dis_weight = 2.2 * gaussian(distance(v1, v2), sigma=sigma, miu=0)
            # if i1 == 0 and i2 == 0:
            #      print(f'第一个色块的相似度为{dis_weight}')

            W1, W2 = np.exp(lamb * w1[i1]), np.exp(lamb * w2[i2])
            dis += (W1 * W2) * dis_weight

            Dis_weight = 2.2 * gaussian(distance(v1, v1), sigma=sigma, miu=0)  # v1是key_color
            Dis += (W1 * W2) * Dis_weight
    #     m = (i1+1)*(i2+1)
    #     print(dis, Dis)
    return dis / Dis

def is_Centers_same(V1,w1,V2,w2,yourScore = 0.75, dist_choise=2,lamb = 5,sigma=1, **dic):
    '''
    V1,w1对应Key color, V通常是一个多Center的列表
    V2,w2对应待分类color
    可调节参数:
    lamb: 取值建议1~10,值越大表示颜色百分比所占权重越大
    sigma: 取值建议0~1.7, 值越小表示相似距离变大(小)时，相似度权重下降(上升)越快, 它的取值与 dist_choise相关:P1(0~3),P2(0~1.7),P3(0~1)
    output: 达到分数返回True 否则返回False
    '''
    # default Para
    printScore = 0

    if 'printScore' in dic:
        printSore = dic['printScore']

    score = simil_score(V1,w1,V2,w2,dist_choise=dist_choise,lamb = lamb,sigma=sigma)
    if printSore == 1:
        print(score)

    if score >= yourScore:
        return True
    else:
        return False


from pypinyin import pinyin, Style, load_phrases_dict

def _pinyinformat(txt):
	'''format pinyin to mtts's format'''
	pinyin_list = pinyin(txt, style=Style.TONE3)
	res = []
	for i in pinyin_list:
		syllable = i[0]
		if not syllable[-1].isdigit():
			syllable = syllable + '0'  # Dacidian format
		assert syllable[-1].isdigit()
		res.append(syllable)
	RES = []
	for i in res:
		RES.append(i[:-1].upper()+'_' + i[-1])
	return RES

def syl2ph_dict():
	syllable_to_phones={}
	with open("pinyin_to_phone.txt",'r', encoding='utf-8') as fi:
		for l in fi.readlines():  # "ZHENG	zh eng"
			cols = l.strip().split('\t')
			assert(len(cols) == 2)
			syllable = cols[0]
			phones = cols[1].split()
			syllable_to_phones[syllable] = phones
	return syllable_to_phones

def py2ph(txt):
	symbol_list = ['sil','<space>','<unk>']
	phone_seq = []
	syllable_to_phones = syl2ph_dict()
	for syllable in _pinyinformat(txt):
		base, tone = syllable.split('_')
		phones = [phn for phn in syllable_to_phones[base]]

		if phones[-1] in symbol_list:
			pass
		else:
			phones[-1] = phones[-1] + '_' + tone
		phone_seq.extend(phones)
	return phone_seq

if __name__ =="__main__":
    txt = "我是光啊"
    print(py2ph(txt))




import itchat
from itchat.content import TEXT
from .Utils import get_userName, _txt
from multiprocessing import Process
from multiprocessing import Pool
import threading
from concurrent.futures import ThreadPoolExecutor
"""
python 由于GIL的缘故，仅使用线程的话，即使在多核cpu的主机上运行也只会使用单个核，
解决这个问题的一个简单方法是在多进程的基础下使用多线程， 多个Python进程有各自独立的GIL锁，互不影响这样依旧能利用多核cpu。 
况且还有协程这个方案可以。
"""
def write(name_list):
    """name_list such as ['filehelper', 'caloi']"""

    lock = threading.Lock()
    def run(nickName):
        print(nickName, ":start")

        @itchat.msg_register([TEXT])
        def write2file(msg):
            userName = get_userName(nickName)[nickName]
            if msg.user.userName == userName:
                _txt(msg)

        lock.acquire()
        itchat.auto_login(hotReload=True, enableCmdQR=2)
        itchat.run()
        lock.release()

    try:
        # --------------thread----------------
        #     t = [threading.Thread(target=run, args=(name,)) for name in name_list]
        #     [i.start() for i in t]
        #     [i.join() for i in t]

        # ------------ 线程池不加锁会导致多次写入信息, 加上线程锁便正常了 --------------------
        with ThreadPoolExecutor(max_workers=len(name_list)) as executor:
            for name in name_list:
                dict1 = {'nickName': name}
                executor.submit(run, **dict1)

    #     ------------process pool------------
    #     with Pool(len(name_list)) as pool:
    #         pool.map(run, name_list)
    except:
        print("try is not successful !")

if __name__ == "__main__":
    name_list = ["filehelper", "caloi"]
    write(name_list)



import sys
import re
from pypinyin import pinyin, Style, load_phrases_dict


consonant_list = ['b', 'p', 'm', 'f', 'd', 't', 'n', 'l', 'g', 'k',
                  'h', 'j', 'q', 'x', 'zh', 'ch', 'sh', 'r', 'z',
                  'c', 's'] # 'y', 'w'


TRANSFORM_DICT = {'ju':'jv', 'qu':'qv', 'xu':'xv', 'zi':'zii',
                    'ci':'cii', 'si':'sii', 'zhi':'zhiii', 
                    'chi':'chii', 'shi':'shii', 'ri':'ri',
                    
                    'quan':'qvan','xuan':'xvan','juan':'jvan',
                    'qun':'qvn','xun':'xvn', 'jun':'jvn',
                  
                    'yuan':'van', 'yue':'ve', 'yun':'vn',
                    'ya':'ia', 'ye':'ie', 'yao':'iao',
                    'yi':'i','yu':'v',
                    'you':'iou', 'yan':'ian', 'yin':'in',
                    'yang':'iang', 'ying':'ing', 'yong':'iong',
                    'yvan':'van', 'yve':'ve', 'yvn':'vn',
                    'wu':'u',
                    'wa':'ua', 'wo':'uo', 'wai':'uai','wei':'uei',
                    'wan':'uan', 'wen':'uen',
                    'weng':'ueng', 'wang':'uang'}

trans_dict = {'uan':'van','iu':'iou','ui':'uei', 'un':'uen', }


def _pre_pinyin_setting():
    ''' fix pinyin error'''
    load_phrases_dict({'嗯':[['ēn']]})

_pre_pinyin_setting()




def _pinyinformat(syllable):
    '''format pinyin to mtts's format''' 
    if not syllable[-1].isdigit():
        syllable = syllable + '5'
    assert syllable[-1].isdigit()
    syl_no_tone = syllable[:-1]
    if syl_no_tone in TRANSFORM_DICT:
        syllable = syllable.replace(syl_no_tone, TRANSFORM_DICT[syl_no_tone])
    return syllable
 
symbol_dict = {',':'sp1', '，':'sp1'}
def _seprate_syllable(syllable):
    '''seprate syllable to consonant + ' ' + vowel '''
    assert syllable[-1].isdigit()

    if syllable[0:2] in consonant_list:
        if syllable[2:-1] in trans_dict.keys():
            syllable = syllable.replace(syllable[2:-1], trans_dict[syllable[2:-1]])
        return syllable[0:2], syllable[2:]

    elif syllable[0] in consonant_list:
        if syllable[1:-1] in trans_dict.keys():
            syllable = syllable.replace(syllable[1:-1], trans_dict[syllable[1:-1]])
        return syllable[0], syllable[1:]
    # guang
    elif syllable[:-1] in symbol_dict.keys():
        return (symbol_dict[syllable[:-1]],)
    else:
        return (syllable,)


def txt2ph(txt):
    phone_list = []

    pinyin_list = pinyin(txt, style = Style.TONE3)
    for item in pinyin_list:
        phone_list.append(_seprate_syllable(_pinyinformat(item[0])))
    return phone_list


import re

def find_match(start, end, S, flag=0):
    """find the string between `start` and `end` of `S`
    flag=0 defaults, means no special specification
    flag options:
        re.I    IGNORECASE， 忽略大小写的匹配模式
        re.M    MULTILINE，多行模式, 改变 ^ 和 $ 的行为
        re.S  　DOTALL，此模式下 '.' 的匹配不受限制，可匹配任何字符，包括换行符，也就是默认是不能匹配换行符
        re.X    VERBOSE，冗余模式， 此模式忽略正则表达式中的空白和#号的注释
    """
    try:
        START = re.search(start, S, flags=flag).span()[1]
        END = re.search(end, S, flags=flag).span()[0]
        return S[START:END]
    except:
        print('Do not match anything.')
        return None

def find_match2(pattern, S, flag=0):
    res = re.search(pattern, S, flags=flag)
    return res.group()


def replace(string, beRepl, repl, count=1):
    """replace `beRepl` to 'repl' from `string` count times.
    if count=0, relplace all."""
    pattern = re.compile(beRepl)
    return pattern.sub(repl, string, count)




import plotly.graph_objects as go
import numpy as np
from guang.Utils.mathFunc import sawtooth

def slider_1D(x, y, width=None, height=None,
              color="#00CED1",
              linewidth=1,
              name="𝜈 = ",
              initial_step=1):
    """ create a 1D slider
    :param a list of array: x:  each menber is a one-dimention array
    :param a list of array: y: a list of f(x)

    Example

        from guang.Utils.mathFunc import sawtooth
        theta = np.linspace(0, 15, 1000)
        N = 10
        x, y = [], []
        for step in range(N):
            x.append(theta)
            y.append(sawtooth(step, theta))
        fig = slider_1D(x=x, y=y)
        fig.show()
    """
    # step 1 :Create a graph window fig, add all the graphs to fig, and set visible to False
    fig = go.Figure()

    for step in range(len(y)):
        fig.add_trace(
            go.Scatter(
                visible=False,

                line=dict(color=color, width=linewidth),  # --------------- 1
                name=name + str(step),                    # --------------- 2
                x=x[step],                                # --------------- 3
                y=y[step]                                 # --------------- 4
            )
        )

    # step 2： initial figure
    initial_step = initial_step  # --------------- 5
    fig.data[initial_step].visible = True

    # step 3: Each time an image is visible=True, the rest is False
    steps = []
    for i in range(len(fig.data)):
        step = dict(
            method="restyle",
            args=["visible", [False] * len(fig.data)],
        )
        step["args"][1][i] = True  # Toggle i'th trace to "visible"
        steps.append(step)

    sliders = [dict(
        active=initial_step,
        currentvalue={"prefix": "frequency: "},  # --------------- 6
        pad={"t": 50},                           # --------------- (7)
        steps=steps
    )]

    fig.update_layout(
        sliders=sliders, 
        width = width,
        height = height
    )
    return fig

if __name__ =="__main__":

    theta = np.linspace(0, 15, 1000)
    N = 10
    x, y = [], []
    for step in range(N):
        x.append(theta)
        y.append(sawtooth(step, theta))
    fig = slider_1D(x=x, y=y, width=600, height=500)
    fig.show()



import guang
from guang.wechat.Utils import * 
import argparse

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-n','--name', help='nike name', type=str, default='桂花香')
    args = parser.parse_args()
    nickName = args.name

    # itchat.auto_login(hotReload=True, enableCmdQR=2)
    reload()
    # print(get_all_info())
    # while d_time(60):
        # msg = dynamic_specified_msg(get_userName(nickName)[nickName])
        # msg = download_file(msg)
        # msg = get_txt(msg)
        


from tensorboard.backend.event_processing import event_accumulator
from guang.Utils.plotly import Multiplots, Subplots
import numpy as np
from glob import glob


event_list = glob('./exp/train_nodev_csmsc_parallel_wavegan.v1/events.out.tfevents.*')

def show(event_list):
    ea=event_accumulator.EventAccumulator(event_list[0]) 
    ea.Reload()

    # for idx, i in enumerate(ea.scalars.Keys()):
    #     print(idx, i)

    val=ea.scalars.Items(ea.scalars.Keys()[11])

    print(len(val))
    steps, loss = {}, {}
    for idx, i in enumerate(ea.scalars.Keys()):
        items = ea.scalars.Items(i)
        steps.setdefault(i, [j.step for j in items])
        loss.setdefault(i, [j.value for j in items])

    # The following works only in jupyterlab
    fig = Multiplots()
    for i in ea.scalars.Keys():
        fig.plot(steps[i], (loss[i]), label=i)

    fig.x_label = 'steps'
    fig.y_label = 'loss'

    fig.show()

